# -*- shell-script -*-
# "show debug" debugger command
#
#   Copyright (C) 2010-2011, 2019 Rocky Bernstein <rocky@gnu.org>
#
#   bashdb is free software; you can redistribute it and/or modify it under
#   the terms of the GNU General Public License as published by the Free
#   Software Foundation; either version 2, or (at your option) any later
#   version.
#
#   bashdb is distributed in the hope that it will be useful, but WITHOUT ANY
#   WARRANTY; without even the implied warranty of MERCHANTABILITY or
#   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with bashdb; see the file COPYING.  If not, write to the Free Software
#   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA.

_Dbg_help_add_sub show debug \
"**show debug**

Show if we are set to debug the debugger." 1

_Dbg_do_show_debug() {
    [[ -n $label ]] && label='debug: '
    _Dbg_show_onoff 'debug' 'Allow debugging the debugger' "$label"
    return 0
}
